package org.dasein.cloud.test;

import java.util.Enumeration;
import java.util.Properties;

import junit.framework.TestCase;

import org.dasein.cloud.CloudException;
import org.dasein.cloud.CloudProvider;
import org.dasein.cloud.InternalException;
import org.dasein.cloud.ProviderContext;
import org.dasein.cloud.services.server.ServerServices;
import org.dasein.cloud.services.server.ServerSize;

public class BaseTestCase extends TestCase {
    
    static public ProviderContext getTestContext(Class<? extends CloudProvider> providerClass) {
        ProviderContext ctx = new ProviderContext();
        Properties props = System.getProperties();
        String publicKey, privateKey;
        
        publicKey = props.getProperty("apiSharedKey");
        privateKey = props.getProperty("apiSecretKey");  
        if( publicKey != null && privateKey != null ) {
            ctx.setAccessKeys(publicKey.getBytes(), privateKey.getBytes());
        }
        
        ctx.setAccountNumber(props.getProperty("accountNumber"));
        ctx.setCloudName(props.getProperty("cloudName"));
        ctx.setEndpoint(props.getProperty("endpoint"));
        ctx.setProviderName(props.getProperty("providerName"));
        ctx.setRegionId(props.getProperty("regionId"));
        
        publicKey = props.getProperty("x509Cert");
        privateKey = props.getProperty("x509Key");    
        if( publicKey != null && privateKey != null ) {
            ctx.setStorageKeys(publicKey.getBytes(), privateKey.getBytes());
        }
        
        Properties custom = new Properties();
        Enumeration<?> names = props.propertyNames();
        
        while( names.hasMoreElements() ) {
            String name = (String)names.nextElement();
            
            if( name.startsWith("test.") ) {
                custom.setProperty(name, props.getProperty(name));
            }
        }
        ctx.setCustomProperties(custom);
        return ctx;
    }
    
    public BaseTestCase(String name) { super(name); }
    
    protected Properties getProperties() {
        return System.getProperties();
    }
    
    protected ProviderContext getTestContext() {
        return getTestContext(ComprehensiveTestSuite.providerClass);
    }
    
    protected String getTestDataCenterId() {
        return getProperties().getProperty("test.dataCenter");
    }
    
    protected String getTestFirewallId() {
        return getProperties().getProperty("test.firewall");
    }
    
    protected String getTestMachineImageId() {
        return getProperties().getProperty("test.machineImage");
    }
    
    protected String getTestProduct() {
        return getProperties().getProperty("test.product");
    }
    
    protected CloudProvider getProvider() throws InstantiationException, IllegalAccessException {
        return ComprehensiveTestSuite.providerClass.newInstance();
    }
    
    protected String launch(CloudProvider provider) throws InternalException, CloudException {
        ServerServices svr = provider.getServerServices();

        if( svr != null ) {
            String[] firewalls = ((getTestFirewallId() == null) ? new String[0] : new String[] { getTestFirewallId() });
        
            return svr.launch(getTestMachineImageId(), new ServerSize(getTestProduct()), getTestDataCenterId(), "dsntest-" + System.currentTimeMillis(), null, null, false, firewalls).getProviderServerId();
        }
        return null;
    }
}
