package org.dasein.cloud.test;

import java.lang.annotation.Annotation;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import org.dasein.cloud.CloudProvider;
import org.junit.Test;

import junit.framework.TestSuite;

public class ComprehensiveTestSuite extends TestSuite {
    static protected Class<? extends CloudProvider> providerClass;
    
    public ComprehensiveTestSuite(Class<? extends CloudProvider> classOfProvider) throws TestConfigurationException {
        super();
        try {
            providerClass = classOfProvider;
            addTests(MinimumFunctionalityTestCase.class);
            addTests(DataCenterTestCase.class);
            
            CloudProvider provider = providerClass.newInstance();
            
            provider.connect(BaseTestCase.getTestContext(providerClass));
            if( provider.getAccessServices() != null ) {
                addTests(AccessTestCase.class);
            }
            if( provider.getAccountingServices() != null ) {
                addTests(AccountingTestCase.class);
            }
            if( provider.getAddressServices() != null ) {
                addTests(IpAddressTestCase.class);
            }
            if( provider.getCdnServices() != null ) {
                addTests(CDNTestCase.class);
            }
            if( provider.getFirewallServices() != null ) {
                addTests(FirewallTestCase.class);
            }
            if( provider.getImageServices() != null ) {
                addTests(ImageTestCase.class);
            }
            if( provider.getKeyValueDatabaseServices() != null ) {
                addTests(KeyValueDatabaseTestCase.class);
            }
            if( provider.getLoadBalancerServices() != null ) {
                addTests(LoadBalancerTestCase.class);
            }
            if( provider.getNotificationsServices() != null ) {
                addTests(NotificationsTestCase.class);
            }
            if( provider.getRelationalDatabaseServices() != null ) {
                addTests(RelationalDatabaseTestCase.class);
            }
            if( provider.getScalingServices() != null ) {
                addTests(ScalingTestCase.class);
            }
            if( provider.getServerServices() != null ) {
                addTests(ServerTestCase.class);
            }
            if( provider.getSnapshotServices() != null ) {
                addTests(SnapshotTestCase.class);
            }
            if( provider.getStorageServices() != null ) {
                addTests(StorageTestCase.class);
            }
            if( provider.getVolumeServices() != null ) {
                addTests(VolumeTestCase.class);
            }
        }
        catch( Throwable t ) {
            throw new TestConfigurationException(t);
        }
    }
    
    private void addTests(Class<? extends BaseTestCase> testClass) throws SecurityException, NoSuchMethodException, IllegalAccessException, IllegalArgumentException, InstantiationException, InvocationTargetException {
        Constructor<? extends BaseTestCase> c = testClass.getConstructor(String.class);
        
        if( c == null ) {
            throw new IllegalAccessException("No single argument string constructor for test class");
        }
        for( Method method : testClass.getDeclaredMethods() ) {
            for( Annotation annotation : method.getAnnotations() ) {
                if( annotation.annotationType().equals(Test.class) ) {
                    addTest(c.newInstance(method.getName()));
                }
            }
        }
    }
}
